/*
 * Main.fx
 *
 * Created on 23.07.2009, 10:36:39
 */

package org.openoffice.setup.FX;

import java.lang.Object;
import java.text.DateFormat;

import javafx.animation.Interpolator;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.animation.transition.FadeTransition;
import javafx.date.DateTime;
import javafx.scene.layout.Container;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.*;
import javafx.scene.paint.Color;
import javafx.scene.Scene;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Line;
import javafx.scene.shape.Rectangle;
import javafx.stage.Stage;

import org.openoffice.setup.FX.controller.DialogController;
import org.openoffice.setup.FX.helpdialog.HelpDialog;
import org.openoffice.setup.InstallData;


/**
 * @author is93718
 */

// There can only be one DialogController and one InstallData object (static context!)
public var installData: InstallData = null;
public var dialogController: DialogController = DialogController{};
public var mainProgressCount: Float = 0;

var innerTitleBox: HBox = HBox{};
var innerMainBox: VBox = VBox{};
var innerButtonBox: HBox = HBox{};

public var dTime: String;
public var titleLogoOpacity = 0.0;

function updateTime(): Void {
    var temp = DateTime { }
    // dTime = DateFormat.getTimeInstance(DateFormat.MEDIUM).format(temp.instant);
    dTime = DateFormat.getTimeInstance(DateFormat.LONG).format(temp.instant);
}

public var dialogName: String = "" on replace {
    innerTitleBox = dialogController.getCurrentDialog().getTitleContent().getTitleBox();
    innerMainBox = dialogController.getCurrentDialog().getMainContent().getMainBox();
    innerButtonBox = dialogController.getCurrentDialog().getButtonContent().getButtonBox();
}

public var contentName: String = "" on replace {
    innerMainBox = dialogController.getCurrentDialog().getMainContent().getMainBox();
}


var titleBox: Container = Container{
                          content: bind innerTitleBox;
                      };

def circle: Circle = Circle {
    centerX:40
    centerY:70
    radius: 25
    fill: Color.AQUA
    opacity: 0.5
}

var rect: Rectangle = Rectangle{
                          width: 200
                          height: 190
                          opacity: 0  // invisible rectangle to set size of mainBox
                      }

var mainBox: Container = Container{
                          // width: 200
                          // height: 200
                          content: bind [rect, circle, innerMainBox];
                      };

var buttonBox: Container = Container{
                          content: bind innerButtonBox;
                      };

def mainGradient: LinearGradient = LinearGradient {
    startX: 1
    startY: 0
    endX: 1
    endY: 1
    stops: [
        Stop {
            offset: 0
            color: Color.LIGHTSLATEGRAY
        },
        Stop {
            offset: 1
            color: Color.LIGHTSTEELBLUE
        }
    ]
}

def line1: Line = Line {
                     startX: 0
                     startY: 0
                     endX: 350
                     endY: 0
                     stroke: Color.BLUE
                     strokeWidth: 1
                 }

def line2: Line = Line {
                     startX: 0
                     startY: 0
                     endX: 350
                     endY: 0
                     stroke: Color.BLUE
                     strokeWidth: 1
                 }

var mainVBox: VBox = VBox {
    layoutX: 20
    layoutY: 20
    spacing: 20
    content: [
        titleBox,
        line1,
        mainBox,
        line2,
        buttonBox
    ]
}

var removeDialog: FadeTransition = FadeTransition {
    // node: titleBox
    node: mainVBox
    duration: 2s
    repeatCount: 1
    autoReverse: false
    fromValue: 1
    toValue: 0
}

var showDialog: FadeTransition = FadeTransition {
    // node: titleBox
    node: mainVBox
    duration: 2s
    repeatCount: 1
    autoReverse: false
    fromValue: 0
    toValue: 1
}


public def helpDialog: HelpDialog = HelpDialog {
    layoutX: 20
    layoutY: 62
    visible: false
    windowWidth: 330
    onConfirm: function() {
        dialogController.enableAllButtons();
    }
}

var stage: Stage;

function run () {

    Timeline {
        repeatCount: Timeline.INDEFINITE
        keyFrames: [
            KeyFrame {
                time: 1s
                canSkip: true
                action: updateTime
            }
        ]
    }.play();

    Timeline {
        autoReverse: true
        repeatCount: Timeline.INDEFINITE
        keyFrames: [
            at (0s) { circle.translateX => 0 }
            at (5s) { circle.translateX => 260.0 tween Interpolator.EASEBOTH }
        ]
    }.play();

    Timeline {
        autoReverse: true
        repeatCount: Timeline.INDEFINITE
        keyFrames: [
            at (0s) { circle.translateY => -40.0 }
            at (4s) { circle.translateY => 95.0 tween Interpolator.EASEBOTH }
        ]
    }.play();

    Timeline {
        autoReverse: true
        repeatCount: Timeline.INDEFINITE
        keyFrames: [
            at (0s) { circle.scaleX => 1 }
            at (0s) { circle.scaleY => 1 }
            at (4.5s) { circle.scaleX => 2 tween Interpolator.EASEBOTH }
            at (4.5s) { circle.scaleY => 2 tween Interpolator.EASEBOTH }
        ]
    }.play();

    Timeline {
        autoReverse: true
        repeatCount: Timeline.INDEFINITE
        keyFrames: [
            at (0s) { titleLogoOpacity => 1.0 }
            at (6s) { titleLogoOpacity => 0.1 tween Interpolator.EASEBOTH }
        ]
    }.play();

    stage = Stage {
        title: "Installationsassistent"
        width: 400
        height: 385
        scene: Scene {
            fill: mainGradient
            content: [ mainVBox, helpDialog ]
        }
    }

    dialogController.readStartDialog();
    dialogController.setCurrentDialog("Start");
    dialogController.getCurrentDialog().doStart();
}
