/*
 * DialogController.fx
 *
 * Created on 23.07.2009, 18:53:40
 */

package org.openoffice.setup.FX.controller;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.dialog.*;
import java.util.HashMap;

import javafx.stage.Alert;

/**
 * @author is93718
 */

public class DialogController {

    public var allDialogs: HashMap = new HashMap();
    public var currentDialog: Dialog;

    public function disableAllButtons(): Void {
        currentDialog.getButtonContent().disableButtonNext = true;
        currentDialog.getButtonContent().disableButtonCancel = true;
        currentDialog.getButtonContent().disableButtonBack = true;
        currentDialog.getButtonContent().disableButtonHelp = true;
    }

    public function enableAllButtons(): Void {
        currentDialog.getButtonContent().disableButtonNext = false;
        currentDialog.getButtonContent().disableButtonCancel = false;
        currentDialog.getButtonContent().disableButtonBack = false;
        currentDialog.getButtonContent().disableButtonHelp = false;
    }

    public function showHelp(): Void {
        Main.helpDialog.heading = currentDialog.titleText;
        // first: Setting displayFile, then: triggerUpdate
        Main.helpDialog.displayFile = currentDialog.getHelpFileName();
        // Alert.inform("Helpfilename: {Main.helpDialog.displayFile}");
        Main.helpDialog.triggerUpdate(currentDialog.getHelpFileName());
        Main.helpDialog.buttonConfirm = currentDialog.getConfirmText();
        Main.helpDialog.makeVisible();
    }

    public function getAllDialogs(): HashMap {
        return allDialogs
    }

    public function getCurrentDialog(): Dialog {
       return currentDialog
    }

    public function listCurrentDialog(): Void {  // for debugging
        println("Debug: Current Dialog: {currentDialog.getName()}")
    }

    public function setCurrentDialog(name: String): Void {
        var obj: Object = allDialogs.get(name);
        if ( obj == null ) {
            var exitController: ExitController = ExitController{};
            exitController.exitProgram("ERROR: currentDialog not initialized");
        }

        if ( obj instanceof Dialog ) {
            currentDialog = (obj as Dialog);
            Main.dialogName = currentDialog.getName();  // triggering the UI update
        } else {
            var exitController: ExitController = ExitController{};
            exitController.exitProgram("ERROR: cast not possible in setCurrentDialog!");
        }
    }

    public function readStartDialog(): Void {
        allDialogs.put("Start", StartDialog{})
    }

    public function readAllDialogs(): Void {
        if (Main.installData.isInstallationMode()) {
            allDialogs.put("Welcome", WelcomeDialog{});
            allDialogs.put("License", LicenseDialog{});
            allDialogs.put("SelectInstallationType", InstallationTypeDialog{});
            allDialogs.put("PathSelection", PathSelectionDialog{});
            allDialogs.put("ModuleSelection", ModuleSelectionDialog{});
            allDialogs.put("ReadyToInstall", ReadyToInstallDialog{});
            allDialogs.put("InstallationOngoing", InstallationOngoingDialog{});
            allDialogs.put("InstallationDone", InstallationDoneDialog{});
        } else {
            allDialogs.put("UninstallWelcome", UninstallWelcomeDialog{});
            allDialogs.put("UninstallType", UninstallTypeDialog{});
            allDialogs.put("UninstallModuleSelection", UninstallModuleSelectionDialog{});
            allDialogs.put("ReadyToUninstall", ReadyToUninstallDialog{});
            allDialogs.put("UninstallationOngoing", UninstallationOngoingDialog{});
            allDialogs.put("UninstallationDone", UninstallationDoneDialog{});
        }
    }

    public function showNewDialog(button: String): Void {
        var meaning: String = currentDialog.getButtonEvent(button);

        if ( meaning.equals("cancel")) {
            var exitController: ExitController = ExitController{};
            exitController.showCancelDialog( currentDialog.getCancelTitle(), currentDialog.getCancelText());
            return
        }

        if ( meaning.equals("help")) {
            showHelp();
            disableAllButtons();
            return
        }

        if ( meaning.equals("exit")) {
            // Main.removeDialog.play();
            FX.exit();
        }

        // Main.removeDialog.play(); // -> Ausfaden does not work
        var repeatDialog = currentDialog.afterShow(meaning.equals("next"));
        // Waiting, until it has vanished

        if ( not repeatDialog ) {
            if ( meaning.equals("next")) {
                setCurrentDialog(currentDialog.getNext());
            } else if ( meaning.equals("back")) {
                setCurrentDialog(currentDialog.getPrevious());
            }
        }

        currentDialog.beforeShow();
        // Main.showDialog.play(); // -> Einfaden does not work
        // titleBox = currentDialog.getTitlePanel().titleBox;
        // disableAllButtons();
        currentDialog.duringShow();
    }

}
