/*
 * WelcomeDialog.fx
 *
 * Created on 23.07.2009, 14:30:27
 */

package org.openoffice.setup.FX.dialog;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.controller.JavaFXTaskBase;
import org.openoffice.setup.FX.dialog.Dialog;
import org.openoffice.setup.FX.frame.*;
import org.openoffice.setup.Installer.Installer;
import org.openoffice.setup.Installer.InstallerFactory;
import org.openoffice.setup.ResourceManager;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.Util.PackageCollector;
import org.openoffice.setup.Util.InfoDir;
import org.openoffice.setup.Util.LogManager;
import org.openoffice.setup.Util.SystemManager;

import java.util.Vector;
import java.io.File;
import java.lang.String;

import javafx.scene.control.ProgressIndicator;
import javafx.scene.paint.Color;

/**
 * @author is93718
 */

public class InstallationOngoingDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des InstallationOngoing-Dialoges";
    override public var titleText = "Installation läuft";

    var helpFileName: String = "String_Helpfile_InstallationOngoing";
    var mainLabel1TextDownload: String = "Downloading packages";
    var mainLabel1Text: String = "Installing packages";
    var mainLabel2Text: String = "";
    var mainLabel3Text: String = "";
    var mainLabel1TextEnd: String = "Finishing Installation";
    // var titleText: String = ResourceManager.getString("String_Prologue1");
    var dialogName: String = "InstallationOngoing";  // only for Info

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.YELLOW;
                                     }

    var mainContent: MainContentInstallationOngoing = MainContentInstallationOngoing {
                                                          label1Text: mainLabel1Text;
                                                          label2Text: mainLabel2Text;
                                                          label3Text: mainLabel3Text;
                                                      }

    var buttonContent: ButtonContent = ButtonContent {
                                           visibleButtonBack: false
                                           visibleButtonNext: false
                                       };

    public override function getNext(): String {
        return "InstallationDone";
    }

    public override function getPrevious(): String {
        return "";
    }

    public override function beforeShow(): Void {
        // creating sorted list of packages to install
        var installPackages: Vector = Main.installData.getInstallPackages();
        var sortedPackages: Vector = new Vector();
        PackageCollector.sortPackages(installPackages, sortedPackages, "install");
        Main.installData.setInstallPackages(sortedPackages);

        if ( Main.installData.isMajorUpgrade() ) {
            // PackageCollector.findOldPackages(installData);
            // Sorting for correct order of uninstallation
            var sortedUninstallPackages: Vector = new Vector();
            PackageCollector.sortPackages(Main.installData.getOldPackages(), sortedUninstallPackages, "uninstall");
            Main.installData.setOldPackages(sortedUninstallPackages);
        }

        // JavaFX Todo
        if ( not Main.installData.getOSType().startsWith("Windows")) {
            var installer: Installer = InstallerFactory.getInstance();
            installer.preInstallationOngoing();
        }

        // Main.progressBar.visible = true;
        // Main.progressIndicator.visible = true;
    }

    function doInstall(): Void {

        var installPackages: Vector = Main.installData.getInstallPackages();
        var removePackages: Vector = Main.installData.getOldPackages();
        var installedPackages: Vector = new Vector();

        LogManager.setCommandsHeaderLine("Installation");
        var installer: Installer;
        // JavaFX Todo
        if ( not Main.installData.getOSType().startsWith("Windows")) {
            installer = InstallerFactory.getInstance();
        }

        if ( Main.installData.isDownloadInstallation() ) {
            // copy all packages into local download directory

            for ( i in [0..installPackages.size()-1] ) {

                if ( installPackages.get(i) instanceof PackageDescription) {
                    var packageData: PackageDescription = installPackages.get(i) as PackageDescription;
                    var localcount = i + 1;
                    var progress = ProgressIndicator.computeProgress(installPackages.size(), localcount);
                    var prog: Integer = (progress * 100) as Integer;
 
                    mainContent.publicLabel1Text = mainLabel1TextDownload;
                    // mainContent.publicLabel1Text = Main.installData.getDownloadTempPath();
                    mainContent.publicLabel2Text = packageData.getPackageName();
                    mainContent.publicLabel3Text = "{prog} %";
                    mainContent.publicProgressCount = progress;
                    mainContent.publicLabelAdTitle = packageData.getAdTitle();
                    mainContent.publicLabelAdText1 = packageData.getAdText();
                    mainContent.publicLabelAdText2 = packageData.getAdText2();
                    FX.deferAction(function() { mainContent.updateContent() }); // update dialog

                    var packageName: String = packageData.getPackageName();
                    var destPath: File = new File(Main.installData.getPackagePath());
                    var subdir: String = "";
                    if (( packageData.getPkgSubdir() != null ) and ( not packageData.getPkgSubdir().equals("") )) {
                        subdir = "{packageData.getPkgSubdir()}/";
                        destPath = new File(destPath, subdir);
                        if ( not destPath.exists() ) {
                            SystemManager.createDirectory(destPath);
                        }
                    }

                    var completeDestFile: File = new File(destPath, packageName);
                    var urlString: String = "{Main.installData.getHtmlPackageRoot()}{subdir}{packageName}";

                    SystemManager.downloadBinaryFile(urlString, completeDestFile.getPath());

                    var repeater: Integer = 1;
                    if ( packageData.getRepeater() != 0 ) {
                        repeater = packageData.getRepeater();
                    }

                    // additional downloads, that need some time
//                    for ( n in [1..repeater] ) {
//                        var newPackageName: String = "{packageName}_{n}";
//                        var newCompleteDestFile: File = new File(destPath, newPackageName);
//                        SystemManager.downloadBinaryFile(urlString, newCompleteDestFile.getPath());
//                    }

                    mainContent.publicLabel2Text = "";

                    if ( Main.installData.isAbortedInstallation() ) {
                        break;
                    }
                }
            }
        }

        for ( i in [0..installPackages.size()-1] ) {

            if ( installPackages.get(i) instanceof PackageDescription) {
                var packageData: PackageDescription = installPackages.get(i) as PackageDescription;
                var localcount = i + 1;
                var progress = ProgressIndicator.computeProgress(installPackages.size(), localcount);
                var prog: Integer = (progress * 100) as Integer;
                // var progress = ProgressIndicator.computeProgress(4, localcount);

                mainContent.publicLabel1Text = mainLabel1Text;
                mainContent.publicLabel2Text = packageData.getPackageName();
                mainContent.publicLabel3Text = "{prog} %";
                mainContent.publicLabelAdTitle = packageData.getAdTitle();
                mainContent.publicLabelAdText1 = packageData.getAdText();
                mainContent.publicLabelAdText2 = packageData.getAdText2();
                mainContent.publicProgressCount = progress;
                FX.deferAction(function() { mainContent.updateContent() }); // update dialog

                // Creating an upgrade process for Solaris packages
                if ( Main.installData.getOSType().equalsIgnoreCase("SunOS") ) {
                    if ( installer.isPackageInstalled(packageData, Main.installData) ) {
                        if ( installer.isInstalledPackageOlder(packageData, Main.installData) ) {
                            packageData.setIgnoreDependsForUninstall(true);
                            installer.uninstallPackage(packageData);
                        } else {
                            continue;  // no downgrading
                        }
                   }
                }

                // JavaFX Todo
                if ( not Main.installData.getOSType().startsWith("Windows")) {
                    installer.installPackage(packageData);
                } else {
                    for ( j in [0..500] ) {
                        println("{i}: {packageData.getPackageName()}: Thread todo {j}");
                    }
                }

                installedPackages.add(packageData);

                mainContent.publicLabel2Text = "";

                if ( Main.installData.isAbortedInstallation() ) {
                    break;
                }
            }
        }

        if ( Main.installData.isMajorUpgrade() ) {
            for ( i in [0..removePackages.size()-1] ) {
                if ( removePackages.get(i) instanceof PackageDescription) {
                    var packageData: PackageDescription = removePackages.get(i) as PackageDescription;
                    // JavaFX Todo
                    if ( not Main.installData.getOSType().startsWith("Windows")) {
                        installer.uninstallPackage(packageData);
                    }
                }
            }
        }

        if ( Main.installData.isAbortedInstallation() ) {
            // undoing the installation
            LogManager.setCommandsHeaderLine("Installation aborted!");
            mainContent.label2Text = ResourceManager.getString("String_UninstallationOngoing1");
            // panel.setStopButtonEnabled(false);

            LogManager.setCommandsHeaderLine("Uninstallation");

            for ( i in [0..installedPackages.size()-1] ) {
                if ( installedPackages.get(i) instanceof PackageDescription) {
                    var packageData: PackageDescription = installedPackages.get(i) as PackageDescription;
                    var progress: Integer = java.lang.Math.round(100/installedPackages.size()) * (i+1);
                    mainContent.progressCount = progress;
                    mainContent.label3Text = packageData.getPackageName();
                    // JavaFX Todo
                    if ( not Main.installData.getOSType().startsWith("Windows")) {
                        installer.uninstallPackage(packageData);
                    }
                }
            }

            // removing already created helper files (admin files)
            var removeFiles: Vector = Main.installData.getRemoveFiles();
            for ( i in [0..removeFiles.size()-1] ) {
                var removeFile: File = new File(removeFiles.get(i) as String);
                SystemManager.deleteFile(removeFile);
            }
        }

        if ( not Main.installData.getOSType().startsWith("Windows")) {
            installer.postInstallationOngoing();
        }
    }


    public override function duringShow(): Void {

        JavaFXTaskBase {
            action: function() {
                // mainContent.showLabel2 = true;
                // localLabelText = "Loading components ...";

                doInstall();
                afterShow(true);

                // finally setting the following dialog automatically
                FX.deferAction(function() {
                        startFollowingDialog();
                    }
                );  // attention: ui change from thread -> deferAction
            }
        }.start();
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        mainContent.publicLabel1Text = mainLabel1TextEnd;
        mainContent.publicLabel2Text = "";

        FX.deferAction(function() { mainContent.updateContent() }); // update dialog

        if ( not Main.installData.isAbortedInstallation() ) {
            if ( not Main.installData.getOSType().startsWith("Windows")) {
                InfoDir.prepareUninstallation();
            } else {
                for ( j in [0..5000] ) {
                    println("AfterShow Todo: {j}");
                }
            }
        }

        return repeatDialog
    }
    
    function startFollowingDialog(): Void {
        Main.dialogController.setCurrentDialog(getNext());
        Main.dialogController.getCurrentDialog().doStart();
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

}
