/*
 * LicenseDialog.fx
 *
 * Created on 23.07.2009, 15:01:07
 */

package org.openoffice.setup.FX.dialog;

import javafx.scene.paint.Color;
import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.frame.*;

import java.util.Vector;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.Calculator;
import org.openoffice.setup.Util.Dumper;
import org.openoffice.setup.Util.ModuleCtrl;
import org.openoffice.setup.Util.PackageCollector;

/**
 * @author is93718
 */

public class InstallationTypeDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des InstallationType-Dialoges";
    override public var titleText = "InstallationType";

    var helpFileName: String = "String_Helpfile_ChooseInstallationType";
    var dialogName: String = "SelectInstallationType"; // only for Debugging
    var mainLabelText: String = "Bitte wählen Sie den Installationstyp aus:";
    var textStandard: String = "Standard Installation";
    var descStandard: String = "Installiert vordefinierte Module";
    var textUserDefined: String = "Benutzerdefinierte Installation";
    var descUserDefined: String = "Wählen Sie selber die Module aus";

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.BLUE;
                                     };

    var mainContent: MainContentInstallationType = MainContentInstallationType {
                                       labelText: mainLabelText;
                                       selection1Text: textStandard;
                                       selection1Desc: descStandard;
                                       selection2Text: textUserDefined;
                                       selection2Desc: descUserDefined;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                       };

    public override function getNext(): String {
        var nextDialog: String = "ReadyToInstall";

        if ( mainContent.selection2.selected ) {
            nextDialog = "ModuleSelection";
        }

        return nextDialog
    }

    public override function getPrevious(): String {
        if ( Main.installData.isRootInstallation() ) {
            if ( Main.installData.hideEula() ) {
                return new String("Welcome");
            } else {
                return new String("License");
            }
        } else {
            return new String("PathSelection");
        }
    }

    public override function beforeShow(): Void {
    }

    public override function duringShow(): Void {
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        if ( isNext ) {

            if ( mainContent.selection1.selected ) {       // standard install
                Main.installData.setIsTypicalInstallation(true);
                Main.installData.setIsCustomInstallation(false);
            } else {                                       // custom install
                Main.installData.setIsTypicalInstallation(false);
                Main.installData.setIsCustomInstallation(true);
            }

            var packageData: PackageDescription = SetupDataProvider.getPackageDescription();

            if ( Main.installData.isTypicalInstallation() ) {
                // If typical selection state values have been saved before,
                // it is now time to restore them

                if ( Main.installData.typicalSelectionStateSaved()) {
                    // System.err.println("Restoring typical selection states");
                    ModuleCtrl.restoreTypicalSelectionStates(packageData);
                }

                if ( Main.installData.logModuleStates() ) {
                    Dumper.logModuleStates(packageData, "ChooseInstallationType: Before setHiddenModuleSettingsInstall");
                }

                // For standard installation type, the hidden modules have to be defined here.
                // Then it is possible to calculate the size of the installed product, to show a warning
                // and to set the repeatDialog value to true
                ModuleCtrl.setHiddenModuleSettingsInstall(packageData);
                // Dumper.dumpInstallPackages(packageData);

                if ( Main.installData.logModuleStates() ) {
                    Dumper.logModuleStates(packageData, "ChooseInstallationType: After setHiddenModuleSettingsInstall");
                }

                // Collecting packages to install
                var installPackages: Vector = new Vector();
                PackageCollector.collectInstallPackages(packageData, installPackages);
                Main.installData.setInstallPackages(installPackages);

                // Check disc space
                // JavaFX Todo
                if ( not Main.installData.getOSType().startsWith("Windows")) {
                    if ( Calculator.notEnoughDiscSpace(Main.installData) ) {
                        repeatDialog = true;
                    }
                }
            }

            // Custom installation type
            if ( Main.installData.isCustomInstallation() ) {
                // Saving typical selection state values, if they are not already saved.
                if ( not Main.installData.typicalSelectionStateSaved()) {
                    // System.err.println("Saving typical selection states");
                    ModuleCtrl.saveTypicalSelectionStates(packageData);
                    Main.installData.setTypicalSelectionStateSaved(true);
                }

                // Setting custom selection state values, if they have been saved before.
                if ( Main.installData.customSelectionStateSaved() ) {
                    // System.err.println("Restoring custom selection states");
                    ModuleCtrl.restoreCustomSelectionStates(packageData);
                }
            }
        }

        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

}
