/*
 * LicenseDialog.fx
 *
 * Created on 23.07.2009, 15:01:07
 */

package org.openoffice.setup.FX.dialog;

import javafx.scene.paint.Color;
import org.openoffice.setup.FX.frame.*;

import javax.swing.tree.TreeSelectionModel;
import org.openoffice.setup.PanelHelper.TreeNodeRenderer;


import org.openoffice.setup.FX.Main;

import org.openoffice.setup.ResourceManager;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.ModuleCtrl;

import java.lang.System;
import java.util.Vector;
import org.openoffice.setup.Util.Calculator;
import org.openoffice.setup.Util.Dumper;
import org.openoffice.setup.Util.Informer;
import org.openoffice.setup.Util.PackageCollector;


import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import org.openoffice.setup.SetupData.DisplayPackageDescription;

// import java.awt.Color;


/**
 * @author is93718
 */

var selection: String;

public class ModuleSelectionDialog extends Dialog, MouseListener, KeyListener, TreeSelectionListener {

    override public var helpFileText = "Dieses ist die Hilfe des ModuleSelection-Dialoges";
    override public var titleText = "Module Selection";

    var helpFileName: String = "String_Helpfile_ChooseComponents";
    var dialogName: String = "ModuleSelection"; // only for Debugging
    var mainLabelText: String = "Bitte wählen Sie die zu installierenden Module aus:";
    var sizeString: String = ResourceManager.getString("String_ChooseComponents4");

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.PINK
                                     };

    var mainContent: MainContentModuleSelection = MainContentModuleSelection {
                                       labelText: mainLabelText;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                       };

    public override function getNext(): String {
        return "ReadyToInstall";
    }

    public override function getPrevious(): String {
        if ( Main.installData.isRootInstallation() ) {
            if ( Main.installData.sameVersionExists() ) {
                if ( Main.installData.hideEula() ) {
                    return new String("Welcome");            
                } else {
                    return new String("License");
                }
            } else {
                return new String("SelectInstallationType");
            }
        } else {
            if ( Main.installData.sameVersionExists() ) {
                return new String("PathSelection");            
            } else {
                return new String("SelectInstallationType");
            }
        }
    }

    public override function beforeShow(): Void {

        // Setting the package size for node modules, that have hidden children
        // -> Java module has three hidden children and 0 byte size

        if ( not Main.installData.moduleSizeSet() ) {
            var packageData: PackageDescription = SetupDataProvider.getPackageDescription();
            ModuleCtrl.setModuleSize(packageData);
            Main.installData.setModuleSizeSet(true);
        }
        
        if ( Main.installData.sameVersionExists() ) {
            var dialogTitle: String = ResourceManager.getString("String_ChooseComponents1_Maintain");
            // Setting dialog title
        }

        var renderer: TreeNodeRenderer = new TreeNodeRenderer();

        mainContent.componentTree.setShowsRootHandles(true);
        mainContent.componentTree.setRootVisible(false);
        mainContent.componentTree.setVisibleRowCount(7);
        mainContent.componentTree.setCellRenderer(renderer);
        mainContent.componentTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        // mainContent.componentTree.setFont(new Font("Arial", Font.ITALIC, 14));
        // mainContent.componentTree.setBackground(java.awt.Color.GRAY);
        // mainContent.componentTree.setOpaque(false);
        // mainContent.componentTree.

        renderer.setTextSelectionColor(java.awt.Color.BLACK);

        mainContent.componentTree.addMouseListener(this);
        mainContent.componentTree.addKeyListener(this);
        mainContent.componentTree.addTreeSelectionListener(this);

        // mainContent.componentTree.setBackground(java.awt.Color.LIGHT_GRAY);
        // mainContent.scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    }

    public override function duringShow(): Void {
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        var packageData: PackageDescription = SetupDataProvider.getPackageDescription();

        if ( isNext ) {

            // Check, if at least one visible module was selected for installation
            Main.installData.setVisibleModulesChecked(false);
            ModuleCtrl.checkVisibleModulesInstall(packageData, Main.installData);

            if ( Main.installData.visibleModulesChecked() ) {

                // Check, if at least one application module was selected for installation
                // (not necessary, if an older product is updated or additional modules are
                // added in maintenance mode).

                var applicationSelected: Boolean = false;
                if ( Main.installData.olderVersionExists() or Main.installData.sameVersionExists() ) {
                    applicationSelected = true;
                } else {
                    Main.installData.setApplicationModulesChecked(false);
                    ModuleCtrl.checkApplicationSelection(packageData, Main.installData);
                    applicationSelected = Main.installData.applicationModulesChecked();
                }

                if ( applicationSelected ) {

                    // Check, if at least one language module was selected for installation
                    // (not necessary, if an older product is updated or additional modules are
                    // added in maintenance mode).

                    var languageSelected: Boolean = false;
                    if ( Main.installData.olderVersionExists() or Main.installData.sameVersionExists() or ( not Main.installData.isMultiLingual())) {
                        languageSelected = true;
                    } else {
                        Main.installData.setLanguageModulesChecked(false);
                        ModuleCtrl.checkLanguageSelection(packageData, Main.installData);
                        languageSelected = Main.installData.languageModulesChecked();
                    }

                    if ( languageSelected ) {

                        // Set module settings for hidden modules.
                        // Then it is possible to calculate the size of the installed product,
                        // to show a warning and to set the repeatDialog value to true

                        if ( Main.installData.logModuleStates() ) {
                            Dumper.logModuleStates(packageData, "ChooseComponentsCtrl: Before setHiddenModuleSettingsInstall");
                        }

                        ModuleCtrl.setHiddenModuleSettingsInstall(packageData);
                        // Dumper.dumpInstallPackages(packageData);

                        if ( Main.installData.logModuleStates() ) {
                            Dumper.logModuleStates(packageData, "ChooseComponentsCtrl: After setHiddenModuleSettingsInstall");
                        }

                        // Collecting packages to install
                        var installPackages: Vector = new Vector();
                        PackageCollector.collectInstallPackages(packageData, installPackages);
                        Main.installData.setInstallPackages(installPackages);

                        // JavaFX Todo
                        if ( not Main.installData.getOSType().startsWith("Windows")) {
                            // Check disc space
                            if ( Calculator.notEnoughDiscSpace(Main.installData) ) {
                                repeatDialog = true;
                            System.err.println("Not enough disc space");
                            }
                        }
                        
                    } else {   // no language modules selected for installation
                        var message: String = "{ResourceManager.getString("String_No_Language_Selected_1")}\n{ResourceManager.getString("String_No_Language_Selected_2")}";
                        var title: String = ResourceManager.getString("String_Change_Selection");
                        Informer.showInfoMessage(message, title);
                        repeatDialog = true;
                    }
                } else {
                    var message: String = "{ResourceManager.getString("String_No_Application_Selected_1")}\n{ResourceManager.getString("String_No_Application_Selected_2")}";
                    var title: String = ResourceManager.getString("String_Change_Selection");
                    Informer.showInfoMessage(message, title);
                    repeatDialog = true;
                }
            } else {  // no modules selected for installation
                var message: String = "{ResourceManager.getString("String_No_Components_Selected_1")}\n{ResourceManager.getString("String_No_Components_Selected_2")}";
                var title: String = ResourceManager.getString("String_Nothing_To_Install");
                Informer.showInfoMessage(message, title);
                repeatDialog = true;
            }
        } else {  // the back button was pressed
            // Saving typical selection state values (always if back button is pressed!).
            // System.err.println("Saving custom selection states");
            ModuleCtrl.saveCustomSelectionStates(packageData);
            Main.installData.setCustomSelectionStateSaved(true);
        }

        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

    // Listener functions

    function updateNode(node: DefaultMutableTreeNode): Void {
        if (node != null) {
            var nodeInfo: DisplayPackageDescription = (node.getUserObject() as DisplayPackageDescription);
            nodeInfo.toggleState(node);

            var model: DefaultTreeModel = (mainContent.componentTree.getModel() as DefaultTreeModel);
            model.nodeChanged(node);

            // The following line was included because of task i78481.
            // In Java 1.6 nodeChanged does not work correctly.
            // model.nodeStructureChanged(node);

            mainContent.labelModuleDescription = nodeInfo.getDescription();
            mainContent.labelModuleSize = "{sizeString} {nodeInfo.getSize()}";
        }
    }

    /**
     * Implement the MouseListener Interface
     */
    public override function mouseClicked(event: MouseEvent): Void  {
    }

    public override function mouseEntered(event: MouseEvent): Void  {
    }

    public override function mouseExited(event: MouseEvent): Void   {
    }

    public override function mousePressed(event: MouseEvent): Void  {
        var selPath: TreePath = mainContent.componentTree.getPathForLocation( event.getX(), event.getY() );
        if ((selPath != null) and (mainContent.componentTree.getPathBounds(selPath).getX() + 20 >= event.getX())) {
            updateNode(selPath.getLastPathComponent() as DefaultMutableTreeNode);
        }
    }

    public override function mouseReleased(e: MouseEvent): Void {
    }

    /**
     * Implement the KeyListener Interface
     */

    public override function keyPressed(event: KeyEvent): Void {
    }

    public override function keyReleased(event: KeyEvent): Void {
    }

    public override function keyTyped(event: KeyEvent): Void {
//        if ( event.getKeyChar() == ' ' ) {
//            var selPath: TreePath = mainContent.componentTree.getAnchorSelectionPath();
//            if ( selPath != null ) {
//                updateNode(selPath.getLastPathComponent() as DefaultMutableTreeNode);
//            }
//        }
    }

    /**
     * Implement the TreeSelectionListener Interface.
     */

    public override function valueChanged(event: TreeSelectionEvent): Void {
        var node: DefaultMutableTreeNode = (mainContent.componentTree.getLastSelectedPathComponent() as DefaultMutableTreeNode);

        if (node == null) {
            mainContent.labelModuleDescription = "";
            mainContent.labelModuleSize = "";
        } else {
            var nodeInfo: DisplayPackageDescription = (node.getUserObject() as DisplayPackageDescription);

            nodeInfo.updateSize(node); // important to set default values for nodes
            var model: DefaultTreeModel = (mainContent.componentTree.getModel() as DefaultTreeModel);
            model.nodeChanged(node);

            mainContent.labelModuleDescription = nodeInfo.getDescription();
            mainContent.labelModuleSize = "{sizeString} {nodeInfo.getSize()}";
        }

    }

    
}
