/*
 * LicenseDialog.fx
 *
 * Created on 23.07.2009, 15:01:07
 */

package org.openoffice.setup.FX.dialog;

import javafx.scene.paint.Color;
import javax.swing.JOptionPane;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.frame.*;
import org.openoffice.setup.Installer.Installer;
import org.openoffice.setup.Installer.InstallerFactory;
import org.openoffice.setup.ResourceManager;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.Controller;
import org.openoffice.setup.Util.Informer;
import org.openoffice.setup.Util.InstallChangeCtrl;
import org.openoffice.setup.Util.LogManager;
import org.openoffice.setup.Util.ModuleCtrl;
import org.openoffice.setup.Util.SystemManager;

import java.io.File;
import java.lang.String;
import java.lang.System;

import javafx.stage.Alert;

/**
 * @author is93718
 */

public class PathSelectionDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des PathSelection-Dialoges";
    override public var titleText = "Path Selection";

    var helpFileName: String = "String_Helpfile_ChooseDirectory";
    var dialogName: String = "PathSelection"; // only for Debugging
    var mainLabelText: String = "Bitte wählen Sie das Installationsverzeichnis aus:";
    var buttonText: String = "Browse ...";
    var defaultDir: String = "";

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.DARKORANGE;
                                     };

    var mainContent: MainContentPathSelection = MainContentPathSelection {
                                       labelText: mainLabelText;
                                       browseButtonText: buttonText;
                                       defaultDirText: defaultDir;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                       };

    public override function getNext(): String {
        if ( Main.installData.olderVersionExists() ) {
            return new String("ReadyToInstall");
        } else if ( Main.installData.sameVersionExists() ) {
            return new String("ModuleSelection");
        } else {
            return new String("SelectInstallationType");
        }
    }

    public override function getPrevious(): String {
        if ( Main.installData.hideEula() ) {
            return new String("Welcome");
        } else {
            return new String("License");
        }
    }

    public override function beforeShow(): Void {

        if ( Main.installData.getInstallDir() == null ) {
            var installDir: String = Main.installData.getDefaultDir();

            // if ( installDir.equalsIgnoreCase("ProgramFilesFolder")) {
            //     if (( System.getProperty("%PROGRAMFILES%") != null ) and ( not System.getProperty("%PROGRAMFILES%").equals("") )) {
            //         installDir = System.getProperty("%PROGRAMFILES%");
            //     }
            // }

            if ( Main.installData.isUserInstallation() ) {
                // System.getenv only supported in Java 1.5, property set in shell script
                // if (( System.getenv("HOME") != null ) && ( ! System.getenv("HOME").equals(""))) {
                //     rootDir = System.getenv("HOME");
                // }
                if (( System.getProperty("user.home") != null ) and ( not System.getProperty("user.home").equals("") )) {
                    installDir = System.getProperty("user.home");
                }
            }

            Main.installData.setInstallDir(installDir);
        }

        mainContent.textInput.text = Main.installData.getInstallDir();
    }

    public override function duringShow(): Void {
    }

    function removeEndingDelimiter(s: String): String {
        var str: String = s;
        str.trim();
        if (( str.endsWith("/")) or ( str.endsWith("\\"))) {
            // removing the last character
            str = str.substring(0, str.length() - 1);
        }
        // mask spaces
        // str.replaceAll(" ", "\\ ");

        return str;
    }

    public override function afterShow(isNext: Boolean): Boolean {
        println("Selected path: {mainContent.textInput.text}");
        var repeatDialog: Boolean = false;
        var dir: String = mainContent.textInput.text;
        dir = removeEndingDelimiter(dir);
        Main.installData.setInstallDir(dir);

        var installDefaultDir: File;
        if ( Main.installData.getOSType().startsWith("Windows") ) {
            installDefaultDir = new File(dir);
        } else {
            installDefaultDir = new File(dir, Main.installData.getDefaultDir());
        }

        Main.installData.setInstallDefaultDir(installDefaultDir.getPath());

        SetupDataProvider.setNewMacro("DIR", installDefaultDir.getPath()); // important for string replacement
        // SetupDataProvider.dumpMacros();

        // Check existence of directory. Try to create, if it does not exist.
        // If successufully created, calculate available disc space

        if ( isNext ) {
            // If the directory exists, is has to be tested, whether the user has write access

            if ( SystemManager.exists_directory(dir) ) {
            	if ( not Controller.createdSubDirectory(dir) ) {
            	    repeatDialog= true;
            	}
            }

            // If the directory does not exist, is has to be tested, whether the user can create it

            if ( not SystemManager.exists_directory(dir)) {
                var title: String = ResourceManager.getString("String_ChooseDirectory4_Question_Title");
                var message: String = null;
                message = ResourceManager.getString("String_ChooseDirectory5_Question_Message");

                var returnValue: Integer = JOptionPane.showConfirmDialog(null, message, title, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);

                if (  returnValue == JOptionPane.YES_OPTION ) {
                    if ( not Controller.createdDirectory(dir) ) {
            	        repeatDialog= true;
            	    }
                }
                else if (returnValue == JOptionPane.NO_OPTION) {
                    repeatDialog = true;
                }
            }

            // Additional tasks, if the directory is okay
            // JavaFX Todo
            if (( not repeatDialog ) and (not Main.installData.getOSType().startsWith("Windows"))) {
                
                // Calculate available disc space
                var discSpace: Integer = SystemManager.calculateDiscSpace(dir);
                Main.installData.setAvailableDiscSpace(discSpace);

                // Is this a new directory, or one that was already set before.
                // In this case, the database does not need to be evaluated again.
                // -> Testing, whether the database path has changed. This can
                // only happen in user installations.
                if ( Main.installData.isUserInstallation() ) {

                    var oldDatabasePath: String = Main.installData.getDatabasePath();

                    // Setting the database path
                    var installer: Installer = InstallerFactory.getInstance();
                    installer.defineDatabasePath();

                    if (( oldDatabasePath == null ) or ( not oldDatabasePath.equals(Main.installData.getDatabasePath()))) {
                        Main.installData.setDatabaseAnalyzed(false);
                        Main.installData.setDatabaseQueried(false);
                    } else {
                        Main.installData.setDatabaseAnalyzed(true);
                    }
                }

                // In installations with user privileges, now it can be controlled,
                // if there are products installed in the selected directory.
                // Therefore the directory selection dialog has to be shown before.
                // In installations with root privileges, this can only be checked,
                // before the destination directory can be set, because it is fix.
                if ( Main.installData.isUserInstallation() ) {
                    LogManager.setCommandsHeaderLine("Checking change installation");
                    InstallChangeCtrl.checkInstallChange(Main.installData);
                    // InstallChangeCtrl.checkInstallChange(data, panel);
                }

                // At this point it is clear, whether the update-Package is installed or not.
                // If it is installed, it is also clear, in which version it is installed.
                // Therefore the following dialog is also defined.

                if ( Main.installData.newerVersionExists() ) {
                    // This can happen only in installation with user privileges.
                    // Installations with root privileges have cancelled installation
                    // already at first dialog.
                    // Possibility to select a new directory, in which no
                    // newer product exists
                    var message: String = "{ResourceManager.getString("String_Newer_Version_Installed_Found")}\n{Main.installData.getInstallDir()}\n{ResourceManager.getString("String_Newer_Version_Database")}: {Main.installData.getDatabasePath()}\n{ResourceManager.getString("String_Newer_Version_Tip")}";
                    var title: String = ResourceManager.getString("String_Error");
                    Informer.showErrorMessage(message, title);

                    repeatDialog = true;
                }

                // If an older version is found, and the update is forbidden, the user
                // can select another installation directory.

                if ( Main.installData.olderVersionExists() and Main.installData.dontUpdate() ) {
                    // This can happen only in installation with user privileges.
                    // Installations with root privileges have cancelled installation
                    // already at first dialog (InstallChangeCtrl.java).
                    // Possibility to select a new directory, in which no
                    // older product exists
                    var message: String = "{ResourceManager.getString("String_Older_Version_Installed_Found")}\n{Main.installData.getInstallDir()}\n{ResourceManager.getString("String_Newer_Version_Database")}: {Main.installData.getDatabasePath()}\n{ResourceManager.getString("String_Newer_Version_Tip")}";
                    var title: String = ResourceManager.getString("String_Error");
                    Informer.showErrorMessage(message, title);

                    repeatDialog = true;
                }

                if ( not repeatDialog ) {

                    if ( not Main.installData.databaseAnalyzed()) {
                        ModuleCtrl.defaultDatabaseAnalysis(Main.installData);
                        Main.installData.setDatabaseAnalyzed(true);
                    }
                }
            }
        }

        return repeatDialog;
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

}
