/*
 * WelcomeDialog.fx
 *
 * Created on 23.07.2009, 14:30:27
 */

package org.openoffice.setup.FX.dialog;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.dialog.Dialog;
import org.openoffice.setup.FX.frame.*;
import javafx.scene.paint.Color;

import java.lang.System;

import org.openoffice.setup.ResourceManager;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.ProductDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.InfoCtrl;

import org.openoffice.setup.Util.Informer;
import org.openoffice.setup.Util.LogManager;

/**
 * @author is93718
 */

public class ReadyToInstallDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des ReadyToInstall-Dialoges";
    override public var titleText = "Ready to install";

    var helpFileName: String = "String_Helpfile_InstallationImminent";
    var mainLabelText: String = "Die Installation kann beginnen";
    var dialogName: String = "ReadyToInstall";  // only for Info
    var htmlInfoText: String = "";

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.GREEN;
                                     }

    // Also using Welcome main content
    var mainContent: MainContentWelcome = MainContentWelcome {
                                              labelText: mainLabelText;
                                          }

    var buttonContent: ButtonContent = ButtonContent {
                                           textButtonNext: "Install >"
                                       };

    public override function getNext(): String {
        return "InstallationOngoing"
    }

    public override function getPrevious(): String {
        if ( Main.installData.isRootInstallation() ) {
            if ( Main.installData.olderVersionExists() ) {
                if ( Main.installData.hideEula() ) {
                    return new String("Welcome");
                } else {
                    return new String("License");
                }
            } else if ( Main.installData.sameVersionExists() ) {
                return new String("ModuleSelection");
            } else {
                if ( Main.installData.isCustomInstallation() ) {
                    return new String("ModuleSelection");
                } else if ( Main.installData.isTypicalInstallation() ) {
                    return new String("SelectInstallationType");
                } else {
                    System.err.println("Error: Unknown installation type!" );
                    return new String("Error");
                }
            }
        } else {
            if ( Main.installData.olderVersionExists() ) {
                return new String("PathSelection");
            } else if ( Main.installData.sameVersionExists() ) {
                return new String("ModuleSelection");
            } else {
                if ( Main.installData.isCustomInstallation() ) {
                    return new String("ModuleSelection");
                } else if ( Main.installData.isTypicalInstallation() ) {
                    return new String("SelectInstallationType");
                } else {
                    System.err.println("Error: Unknown installation type!" );
                    return new String("Error");
                }
            }
        }
    }

    public override function beforeShow(): Void {
        
        var productData: ProductDescription = SetupDataProvider.getProductDescription();
        var packageData: PackageDescription = SetupDataProvider.getPackageDescription();
        // Dumper.dumpPackageSettings(packageData);
        htmlInfoText = InfoCtrl.setHtmlFrame("header", htmlInfoText);
        htmlInfoText = InfoCtrl.setReadyToInstallInfoText(productData, htmlInfoText);
        htmlInfoText = InfoCtrl.setReadyToInstallInfoText(packageData, htmlInfoText);
        htmlInfoText = InfoCtrl.setHtmlFrame("end", htmlInfoText);

        // Update mode
        if ( Main.installData.olderVersionExists() ) {
            var dialogTitle: String = ResourceManager.getString("String_InstallationImminent1_Update");
            // setting new dialog title
        }
    }

    public override function duringShow(): Void {
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        if ( isNext ) {

            // determining the packagePath
            if ( Main.installData.getPackagePath() != null ) {
                var log: String = "<b>Packages path:</b>{Main.installData.getPackagePath()}<br>";
                LogManager.addLogfileComment(log);
            } else {
                var message: String = ResourceManager.getString("String_InstallationOngoing_PackagePath_Not_Found");
                var title: String = ResourceManager.getString("String_Error");
                Informer.showErrorMessage(message, title);
                var log: String = "<b>Error: No path for packages exists!</b><br>";
                LogManager.addLogfileComment(log);
                repeatDialog = true;
            }
        }

        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

}
