/*
 * WelcomeDialog.fx
 *
 * Created on 23.07.2009, 14:30:27
 */

package org.openoffice.setup.FX.dialog;

import org.openoffice.setup.FX.controller.ExitController;
import org.openoffice.setup.FX.controller.JavaFXTaskBase;
import org.openoffice.setup.FX.dialog.Dialog;
import org.openoffice.setup.FX.frame.*;
import org.openoffice.setup.FX.Main;
import org.openoffice.setup.InstallData;
import org.openoffice.setup.Util.SystemManager;

import java.io.File;
import java.lang.Class;
import java.lang.System;
import java.lang.Exception;
import java.lang.String;
import java.net.URI;
import java.net.URL;
import java.util.Vector;

import javafx.scene.control.ProgressIndicator;
import javafx.scene.paint.Color;

import javafx.stage.Alert;

/**
 * @author is93718
 */

public class StartDialog extends Dialog {

    // ATTENTION: No localization allowed here, because all important files
    // are loaded, when this dialog is displayed
    override public var helpFileText = "Dieses ist die Hilfe des Start-Dialoges";
    override public var titleText = "Starting Installation";

    var helpFileName: String = "String_Helpfile_Prologue";
    var mainLabelText: String = "Preparing installation";
    var mainLabel2Text: String = "Loading components ...";
    var dialogName: String = "Start";  // only for Info
    var nextDialog: String = "";

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.CHOCOLATE;
                                     }

    var mainContent: MainContentStart = MainContentStart {
                                            labelText: mainLabelText;
                                            label2Text: mainLabel2Text;
                                        }

    var buttonContent: ButtonContent = ButtonContent {
                                           visibleButtonBack: false
                                           visibleButtonNext: false
                                           visibleButtonHelp: false
                                           visibleButtonCancel: false
                                       };

    function setNextDialog(dialog: String): Void {
        nextDialog = dialog;
    }

    public override function getNext(): String {
        return nextDialog;
    }

    public override function getPrevious(): String {
        return "";
    }

    public override function beforeShow(): Void {
    }

    public override function duringShow(): Void {
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;
        return repeatDialog
    }

    public override function doStart(): Void {

        JavaFXTaskBase {
            action: function() {
                // Two very important todos, that have to be done at
                // program start: Initialzing installData and read all dialogs!
                Main.installData = InstallData.getInstance();
                Main.installData.setResourceRoot(getResourceRoot());
                Main.installData.setInstallationMode();
                Main.installData.setSolarisUserInstall();
                Main.installData.setHtmlFileExistence();
                Main.installData.setBidiSupport();
                Main.installData.setIsFXInstallation(true);

                Main.dialogController.readAllDialogs();
                if ( Main.installData.isInstallationMode() ) {
                    setNextDialog("Welcome");
                } else {
                    setNextDialog("UninstallWelcome");
                }

                FX.deferAction(function() { Main.dialogController.showNewDialog("buttonNext") });
            }
        }.start();
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

    /* the installation root is where the classes reside */
    function getResourceRoot(): File {
        var dir: File = null;
        try {
            var c: Class = Class.forName("org.openoffice.setup.ResourceManager");
            var url: URL  = c.getResource("setupfiles.properties");

            var urlString: String= url.toString();

            // println("URLString: {urlString}");

            if (urlString.startsWith("jar:")) {
                /* ResourceManager.class resides in a jar file. Strip it down to the "file:" part */
                urlString = urlString.substring(4, urlString.lastIndexOf("!"));
            } else {
                /* ResourceManager.class resides in a directory tree. */
                urlString = urlString.substring(0, urlString.lastIndexOf("/org/openoffice/setup/setupfiles.properties"));
            }

            println("URL: {urlString}");
            // Alert.inform(urlString);

            if ( urlString.startsWith("http:") ) {

                var jarfilename: String = urlString;
                jarfilename = urlString.substring(urlString.lastIndexOf("/") + 1);
                Main.installData.setJarFileName(jarfilename); // saving for later download of jar file

                var tempdir: String = System.getProperty("java.io.tmpdir");
                var installsetPathFileName: String = Main.installData.getTempInstallSetFileName();
                var installsetPathFileFile: File = new File(tempdir, installsetPathFileName);
                var installsetPath: String = null;

                var isOnlineInstallation: Boolean = true;

                // This can be a Java WebStart CD Installation. In this case
                // the installsetPathFileFile exists in the tmp-directory.

                if ( installsetPathFileFile.exists() ) {

                    var tmpFileContent: Vector = SystemManager.readCharFileVector(installsetPathFileFile.getPath());
                    installsetPath = tmpFileContent.get(0) as String;

                    println("Checking install set path: {installsetPath}");

                    // dir = new File(new URI(installsetPath)); -> required for file url
                    dir = new File(installsetPath);

                    // checking existence of installation set (jar file)
                    var jarFile: File = new File(dir, jarfilename);

                    if ( jarFile.exists() ) {
                        // -> Saving the path for later usage (update)
                        Main.installData.setTempInstallSetPath(installsetPath);
                        isOnlineInstallation = false;
                        println("Valid installation set. File exists: {jarFile.getPath()}");
                        println("Java WebStart Application offline");
                    } else {
                        // this seems to be an invalid installation path
                        isOnlineInstallation = true;
                        println("Invalid installation set. File does not exist: {jarFile.getPath()}");
                    }

                    installsetPathFileFile.delete(); // not sure, how about update without calling setup?
                }

                // if (( not installsetPathFileFile.exists() ) and ( isOnlineInstallation )) {
                if ( isOnlineInstallation ) {
                    System.err.println("WebStart Application online");
                    urlString = urlString.substring(0, urlString.lastIndexOf("/"));  // removing xyz.jar
                    Main.installData.setHtmlResourceRoot(urlString); // saving for later download of packages
                    Main.installData.setIsDownloadInstallation(true);
                    // Downloading folder "installdata" to local temp directory
                    var downloadTempPath: String = SystemManager.createUniqueTempDir(Main.installData.getDownloadDirName(), Main.installData.getProcessId());
                    Main.installData.setDownloadTempPath(downloadTempPath);
                    dir = new File(downloadTempPath);
                    // not starting the download, can take some time
                    // SystemManager.downloadInstallData(dir, Main.installData.getHtmlResourceRoot(), Main.installData.getInstallDirName());
                    downloadInstallData(dir, Main.installData.getHtmlResourceRoot(), Main.installData.getInstallDirName());
                    // Download has to happen here, so that progress can be displayed
                }
            } else {
                dir = new File(new URI(urlString)); // includes jar file
                dir = dir.getParentFile();
            }

        } catch (ex: Exception) {
            /* handle URISyntaxException and ClassNotFoundException */
            ex.printStackTrace();
            System.exit(1);
        }

        if ( dir != null ) {
            // System.err.println("Resource Root: " + dir.getPath());
        } else {
            System.err.println("No resource root found!");
        }

        return dir;
    }

    function downloadInstallData(downloadTempPath: File, urlString: String, subdir: String): Void {

        mainContent.publicVisibility = true;
        mainContent.publicLabelText = "Loading installation info";
        FX.deferAction(function() { mainContent.updateContent() }); // update dialog

        var installDataPath: File = new File(downloadTempPath, subdir);
        var xpdDownloadPath: File = new File(installDataPath, "xpd");
        var htmlDownloadPath: File = new File(installDataPath, "html");
        var imagesDownloadPath: File = new File(installDataPath, "images");

        SystemManager.createDirectory(installDataPath);
        SystemManager.createDirectory(xpdDownloadPath);
        SystemManager.createDirectory(htmlDownloadPath);
        SystemManager.createDirectory(imagesDownloadPath);

        // "index.txt" contains all information of files in directory "installdata"

        var webIndexFile: String = "{urlString}/installdata/index.txt";
        var localIndexFile: File = new File(installDataPath, "index.txt");

        SystemManager.downloadTextFile(webIndexFile, localIndexFile.getPath());

        if ( not localIndexFile.exists() ) {
            var exitController: ExitController = ExitController{};
            exitController.exitProgramAlert("ERROR", "Failed to download {webIndexFile} to {localIndexFile.getPath()}");
        }

        // Reading file index.txt
        var allFiles: Vector = SystemManager.readCharFileVector(localIndexFile.getPath());

        for ( i in [0..allFiles.size()-1] ) {
            var oneFile: String = allFiles.get(i) as String;
            oneFile = oneFile.trim();
            if (( oneFile.startsWith("#")) or ( oneFile.equals(""))) { continue; }

            var webFileName: String = "{urlString}/installdata/{oneFile}";
            var localFileName: File = new File(installDataPath, oneFile);

            var isBinary: Boolean = false;
            if ( oneFile.endsWith("gif") ) { isBinary = true; }

            // html files cannot be downloaded correctly. Therefore they are named
            // .html.txt and then downloaded as .html
            if ( localFileName.getPath().endsWith(".html.txt") ) {
                var newFileName: String = localFileName.getPath().substring(0, localFileName.getPath().lastIndexOf("."));
                localFileName = new File(newFileName);
            }

            // displaying progress in progressbar
            var localcount = i + 1;
            var progress = ProgressIndicator.computeProgress(allFiles.size(), localcount);
            mainContent.publicProgressCount = progress;
            mainContent.publicLabel2Text = oneFile;
            FX.deferAction(function() { mainContent.updateContent() }); // update dialog

            if ( isBinary ) {
                SystemManager.downloadBinaryFile(webFileName, localFileName.getPath());
            } else {
                SystemManager.downloadTextFile(webFileName, localFileName.getPath());
            }
        }
    }

}
