/*
 * LicenseDialog.fx
 *
 * Created on 23.07.2009, 15:01:07
 */

package org.openoffice.setup.FX.dialog;

import javafx.scene.paint.Color;
import org.openoffice.setup.FX.frame.*;

import javax.swing.tree.TreeSelectionModel;
import org.openoffice.setup.PanelHelper.TreeNodeRenderer;


import org.openoffice.setup.FX.Main;

import org.openoffice.setup.ResourceManager;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.ModuleCtrl;

import java.lang.System;
import java.util.Vector;
import org.openoffice.setup.Util.Calculator;
import org.openoffice.setup.Util.Dumper;
import org.openoffice.setup.Util.Informer;
import org.openoffice.setup.Util.PackageCollector;


import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import org.openoffice.setup.SetupData.DisplayPackageDescription;


/**
 * @author is93718
 */

var selection: String;

public class UninstallModuleSelectionDialog extends Dialog, MouseListener, KeyListener, TreeSelectionListener {

    override public var helpFileText = "Dieses ist die Hilfe des Uninstall-ModuleSelection-Dialoges";
    override public var titleText = "Uninstallation Module Selection";

    var helpFileName: String = "String_Helpfile_ChooseUninstallationComponents";
    var dialogName: String = "UninstallModuleSelection"; // only for Debugging
    var mainLabelText: String = "Bitte wählen Sie die zu deinstallierenden Module aus:";
    var sizeString: String = ResourceManager.getString("String_ChooseComponents4");

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.PINK
                                     };

    var mainContent: MainContentModuleSelection = MainContentModuleSelection {
                                       labelText: mainLabelText;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                       };

    public override function getNext(): String {
        return "ReadyToUninstall";
    }

    public override function getPrevious(): String {
        return "UninstallType";
    }

    public override function beforeShow(): Void {

        mainContent.componentTree.setShowsRootHandles(true);
        mainContent.componentTree.setRootVisible(false);
        mainContent.componentTree.setVisibleRowCount(7);
        mainContent.componentTree.setCellRenderer(new TreeNodeRenderer());
        mainContent.componentTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        // mainContent.componentTree.setFont(new Font("Arial", Font.ITALIC, 14));
        // mainContent.componentTree.setBackground(java.awt.Color.GRAY);
        // mainContent.componentTree.setOpaque(false);

        mainContent.componentTree.addMouseListener(this);
        mainContent.componentTree.addKeyListener(this);
        mainContent.componentTree.addTreeSelectionListener(this);


        // mainContent.scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    }

    public override function duringShow(): Void {
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        var packageData: PackageDescription = SetupDataProvider.getPackageDescription();

        if ( isNext ) {

            if ( Main.installData.logModuleStates() ) {
                Dumper.logModuleStates(packageData, "Choose UninstallationComponents: Before checkVisibleModulesUninstall");
            }

            // Check, if at least one visible module is selected for uninstallation
            Main.installData.setVisibleModulesChecked(false);
            ModuleCtrl.checkVisibleModulesUninstall(packageData, Main.installData);

            if ( not Main.installData.visibleModulesChecked() ) {
                var message: String = "{ResourceManager.getString("String_No_Uninstallcomponents_Selected_1")}\n{ResourceManager.getString("String_No_Uninstallcomponents_Selected_2")}";
                var title: String = ResourceManager.getString("String_Nothing_To_Uninstall");
                Informer.showInfoMessage(message, title);
                repeatDialog = true;
            } else {
                // Check, if all visible modules are selected for uninstallation.
                // Then this shall be handled as complete uninstallation
                // -> The responsible value is InstallData.isMaskedCompleteUninstallation
                Main.installData.setMaskedCompleteUninstallation(true);
                ModuleCtrl.checkMaskedCompleteUninstallation(packageData, Main.installData);

                // If this is not a complete uninstallation, at least one language
                // module or one application module has to be installed.

                if ( not Main.installData.isMaskedCompleteUninstallation() ) {

                    Main.installData.setApplicationModulesChecked(false);
                    ModuleCtrl.checkApplicationModulesUninstall(packageData, Main.installData);

                    if ( not Main.installData.applicationModulesChecked() ) {

                        var message: String = "{ResourceManager.getString("String_All_Applicationcomponents_Selected_1")}\n{ResourceManager.getString("String_All_Applicationcomponents_Selected_2")}";
                        var title: String = ResourceManager.getString("String_Change_Selection");
                        Informer.showInfoMessage(message, title);
                        repeatDialog = true;
                    } else {
                        if ( Main.installData.isMultiLingual()) {
                            Main.installData.setLanguageModulesChecked(false);
                            ModuleCtrl.checkLanguageModulesUninstall(packageData, Main.installData);

                            if ( not Main.installData.languageModulesChecked() ) {

                                var message: String = "{ResourceManager.getString("String_All_Languagecomponents_Selected_1")}\n{ResourceManager.getString("String_All_Languagecomponents_Selected_2")}";
                                var title: String = ResourceManager.getString("String_Change_Selection");
                                Informer.showInfoMessage(message, title);
                                repeatDialog = true;
                            }
                        }
                    }
                }
            }
        } else {  // the back button was pressed
            // Saving typical selection state values (always if back button is pressed!).
            ModuleCtrl.saveCustomSelectionStates(packageData);
            Main.installData.setCustomSelectionStateSaved(true);
        }

        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

    // Listener functions

    function updateNode(node: DefaultMutableTreeNode): Void {
        if (node != null) {
            var nodeInfo: DisplayPackageDescription = (node.getUserObject() as DisplayPackageDescription);
            nodeInfo.toggleState(node);

            var model: DefaultTreeModel = (mainContent.componentTree.getModel() as DefaultTreeModel);
            model.nodeChanged(node);

            // The following line was included because of task i78481.
            // In Java 1.6 nodeChanged does not work correctly.
            // model.nodeStructureChanged(node);

            mainContent.labelModuleDescription = nodeInfo.getDescription();
            mainContent.labelModuleSize = "{sizeString} {nodeInfo.getSize()}";
        }
    }

    /**
     * Implement the MouseListener Interface
     */
    public override function mouseClicked(event: MouseEvent): Void  {
    }

    public override function mouseEntered(event: MouseEvent): Void  {
    }

    public override function mouseExited(event: MouseEvent): Void   {
    }

    public override function mousePressed(event: MouseEvent): Void  {
        var selPath: TreePath = mainContent.componentTree.getPathForLocation( event.getX(), event.getY() );
        if ((selPath != null) and (mainContent.componentTree.getPathBounds(selPath).getX() + 20 >= event.getX())) {
            updateNode(selPath.getLastPathComponent() as DefaultMutableTreeNode);
        }
    }

    public override function mouseReleased(e: MouseEvent): Void {
    }

    /**
     * Implement the KeyListener Interface
     */

    public override function keyPressed(event: KeyEvent): Void {
    }

    public override function keyReleased(event: KeyEvent): Void {
    }

    public override function keyTyped(event: KeyEvent): Void {
//        if ( event.getKeyChar() == ' ' ) {
//            var selPath: TreePath = mainContent.componentTree.getAnchorSelectionPath();
//            if ( selPath != null ) {
//                updateNode(selPath.getLastPathComponent() as DefaultMutableTreeNode);
//            }
//        }
    }

    /**
     * Implement the TreeSelectionListener Interface.
     */

    public override function valueChanged(event: TreeSelectionEvent): Void {
        var node: DefaultMutableTreeNode = (mainContent.componentTree.getLastSelectedPathComponent() as DefaultMutableTreeNode);

        if (node == null) {
            mainContent.labelModuleDescription = "";
            mainContent.labelModuleSize = "";
        } else {
            var nodeInfo: DisplayPackageDescription = (node.getUserObject() as DisplayPackageDescription);

            nodeInfo.updateSize(node); // important to set default values for nodes
            var model: DefaultTreeModel = (mainContent.componentTree.getModel() as DefaultTreeModel);
            model.nodeChanged(node);

            mainContent.labelModuleDescription = nodeInfo.getDescription();
            mainContent.labelModuleSize = "{sizeString} {nodeInfo.getSize()}";
        }

    }

    
}
