/*
 * LicenseDialog.fx
 *
 * Created on 23.07.2009, 15:01:07
 */

package org.openoffice.setup.FX.dialog;

import javafx.scene.paint.Color;
import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.frame.*;

import java.util.Vector;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.Calculator;
import org.openoffice.setup.Util.Dumper;
import org.openoffice.setup.Util.ModuleCtrl;
import org.openoffice.setup.Util.PackageCollector;

/**
 * @author is93718
 */

public class UninstallTypeDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des UninstallationType-Dialoges";
    override public var titleText = "UninstallationType";

    var helpFileName: String = "String_Helpfile_ChooseUninstallationType";
    var dialogName: String = "UninstallationType"; // only for Debugging
    var mainLabelText: String = "Bitte wählen Sie den Deinstallationstyp aus:";
    var textStandard: String = "Vollständige Deinstallation";
    var descStandard: String = "Deinstalliert alle Module";
    var textUserDefined: String = "Benutzerdefinierte Deinstallation";
    var descUserDefined: String = "Wählen Sie selber die Module aus";

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.BLUE;
                                     };

    var mainContent: MainContentInstallationType = MainContentInstallationType {
                                       labelText: mainLabelText;
                                       selection1Text: textStandard;
                                       selection1Desc: descStandard;
                                       selection2Text: textUserDefined;
                                       selection2Desc: descUserDefined;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                       };

    public override function getNext(): String {
        var nextDialog: String = "ReadyToUninstall";

        if ( mainContent.selection2.selected ) {
            nextDialog = "UninstallModuleSelection";
        }

        return nextDialog
    }

    public override function getPrevious(): String {
        return new String("UninstallWelcome");
    }

    public override function beforeShow(): Void {
    }

    public override function duringShow(): Void {
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        if ( isNext ) {

            if ( mainContent.selection1.selected ) {       // standard install
                Main.installData.setIsTypicalInstallation(true);
                Main.installData.setIsCustomInstallation(false);
            } else {                                       // custom install
                Main.installData.setIsTypicalInstallation(false);
                Main.installData.setIsCustomInstallation(true);
            }

            var packageData: PackageDescription = SetupDataProvider.getPackageDescription();

            // Typical uninstallation type
            if ( Main.installData.isTypicalInstallation() ) {
                // If typical selection state values have been saved before,
                // it is now time to restore them

                if ( Main.installData.typicalSelectionStateSaved()) {
                    ModuleCtrl.restoreTypicalSelectionStates(packageData);
                }
            }

            // Custom uninstallation type
            if ( Main.installData.isCustomInstallation() ) {
                // Saving typical selection state values, if they are not already saved.
                if ( not Main.installData.typicalSelectionStateSaved()) {
                    ModuleCtrl.saveTypicalSelectionStates(packageData);
                    Main.installData.setTypicalSelectionStateSaved(true);
                }

                // Setting custom selection state values, if they have been saved before.
                if ( Main.installData.customSelectionStateSaved() ) {
                    ModuleCtrl.restoreCustomSelectionStates(packageData);
                }
            }
        }

        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

}
