/*
 * WelcomeDialog.fx
 *
 * Created on 23.07.2009, 14:30:27
 */

package org.openoffice.setup.FX.dialog;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.dialog.Dialog;
import org.openoffice.setup.FX.controller.JavaFXTaskBase;
import org.openoffice.setup.FX.frame.*;
import javafx.scene.paint.Color;

import org.openoffice.setup.ResourceManager;

import java.lang.String;

import java.lang.System;
import org.openoffice.setup.Installer.Installer;
import org.openoffice.setup.Installer.InstallerFactory;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.Dumper;
import org.openoffice.setup.Util.ModuleCtrl;

import org.openoffice.setup.Util.Informer;
import org.openoffice.setup.Util.LogManager;

import java.io.File;
import java.util.HashMap;
import java.util.Vector;
import org.openoffice.setup.Util.Converter;
import org.openoffice.setup.Util.SystemManager;



/**
 * @author is93718
 */

public class UninstallWelcomeDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des UninstallWelcome-Dialoges";
    override public var titleText = ResourceManager.getString("String_Prologue1");

    var helpFileName: String = "String_Helpfile_UninstallationPrologue";
    var mainLabelText: String = "Herzlich Willkommen zur Deinstallation dieses Produktes";
    var mainLabel2Text: String = "Loading components ...";
    var dialogName: String = "Welcome";  // only for Info

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.YELLOW;
                                     }

    var mainContent: MainContentWelcome = MainContentWelcome {
                                       labelText: mainLabelText;
                                       label2Text: mainLabel2Text;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                           visibleButtonBack: false
                                       };

    public override function getNext(): String {
        return "UninstallType";
    }

    public override function getPrevious(): String {
        return "";
    }

    public override function beforeShow(): Void {
    }

    public override function duringShow(): Void {
        Main.installData.setIsFXInstallation(true);
        buttonContent.visibleButtonNext = false;
        buttonContent.visibleButtonCancel = false;

        JavaFXTaskBase {
            action: function() {

                preUninstall();
       
                mainContent.publicLabelText = "Welcome";
                mainContent.publicLabel2Text = "All components loaded";
                buttonContent.publicVisibleButtonNext = true;
                buttonContent.publicVisibleButtonCancel = true;

                FX.deferAction(function() { mainContent.updateContent() }); // update dialog
                FX.deferAction(function() { buttonContent.updateContent() }); // update dialog
            }
        }.start();
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;
        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

    function preUninstall(): Void {

        if ( not Main.installData.databaseAnalyzed() ) {

            // now it is time to read the infoFile in directory "installData.getInfoRoot()"
            readInfoFile();

            // controlling the installation privileges. Are the original installation privileges
            // identical with the current deinstallation privileges?
            checkUninstallPrivileges();

            var packageData: PackageDescription = SetupDataProvider.getPackageDescription();

            var installer: Installer;
            // JavaFX Todo
            if ( not Main.installData.getOSType().startsWith("Windows")) {
                installer = InstallerFactory.getInstance();
                installer.preUninstall(packageData);
            }

            // searching in the database for already installed packages
            LogManager.setCommandsHeaderLine("Analyzing system database");

            // JavaFX Todo
            if ( not Main.installData.getOSType().startsWith("Windows")) {
                ModuleCtrl.setDatabaseSettings(packageData, Main.installData, installer);
            }

            Main.installData.setDatabaseAnalyzed(true);
            ModuleCtrl.setDontUninstallFlags(packageData);

        if ( Main.installData.isRootInstallation() ) { ModuleCtrl.setDontUninstallUserInstallOnylFlags(packageData); }
            ModuleCtrl.setParentDefaultModuleSettings(packageData);
        }
    }

    function readInfoFile(): Void {
        var map: HashMap = getInfoFileData();
        setNewInstallData(map);
        Dumper.dumpNewInstallData();
    }

    function checkUninstallPrivileges(): Void {
        // data.setStoredInstallationPrivileges((String)map.get("InstallationPrivileges"));
        var originalPrivileges: String = Main.installData.getStoredInstallationPrivileges();
        var currentPrivileges: String = Main.installData.getInstallationPrivileges();
        if ( not currentPrivileges.equalsIgnoreCase(originalPrivileges) ) {
            // aborting installation with error message
            if ( currentPrivileges.equalsIgnoreCase("root")) {
                var message: String = ResourceManager.getString("String_UninstallationPrologue_Wrong_Privileges_Current_Root");
                var title: String = ResourceManager.getString("String_Error");
                Informer.showErrorMessage(message, title);
                var log: String = "<b>Error: Wrong uninstallation privileges (currently Root)!</b><br>";
                System.err.println(log);
                // LogManager.addLogfileComment(log);
            } else {
                var message: String = ResourceManager.getString("String_UninstallationPrologue_Wrong_Privileges_Current_User");
                var title: String = ResourceManager.getString("String_Error");
                Informer.showErrorMessage(message, title);
                var log: String = "<b>Error: Wrong uninstallation privileges (currently User)!</b><br>";
                System.err.println(log);
                // LogManager.addLogfileComment(log);
            }
            System.exit(1);
        }

    }

    function getInfoFileData(): HashMap {
        var infoRootDir: File = Main.installData.getInfoRoot();
        var infoFilename: String = "infoFile";
        var infoFile: File = new File(infoRootDir, infoFilename);
        var infoFileContent: Vector = SystemManager.readCharFileVector(infoFile.getPath());
        var map: HashMap = Converter.convertVectorToHashmap(infoFileContent);

        // for (int i = 0; i < infoFileContent.size(); i++) {
        //    System.out.println(infoFileContent.get(i));
        // }

        // Iterator m = map.entrySet().iterator();
        // while ( m.hasNext() ) {
        //     Map.Entry entry = (Map.Entry) m.next();
        //     System.out.println( "MAP:" + entry.getKey() + ":" + entry.getValue() );
        // }

        return map;
    }

    function setNewInstallData(map: HashMap): Void {
        // adding information to installData
        Main.installData.setPackagePath(map.get("PackagePath") as String);
        Main.installData.setAdminFileNameReloc(map.get("AdminFileReloc") as String);
        Main.installData.setAdminFileNameRelocNoDepends(map.get("AdminFileRelocNoDepends") as String);
        Main.installData.setAdminFileNameNoReloc(map.get("AdminFileNoReloc") as String);
        Main.installData.setAdminFileNameNoRelocNoDepends(map.get("AdminFileNoRelocNoDepends") as String);
        Main.installData.setDatabasePath(map.get("DatabasePath") as String);
        Main.installData.setInstallDir(map.get("InstallationDir") as String);
        Main.installData.setStoredInstallationPrivileges(map.get("InstallationPrivileges") as String);
        Main.installData.setGetUidPath(map.get("GetUidFile") as String);
    }


}
