/*
 * WelcomeDialog.fx
 *
 * Created on 23.07.2009, 14:30:27
 */

package org.openoffice.setup.FX.dialog;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.dialog.Dialog;
import org.openoffice.setup.FX.frame.*;
import javafx.scene.paint.Color;



import org.openoffice.setup.Installer.InstallerFactory;

import java.util.Vector;
import org.openoffice.setup.Util.PackageCollector;

import org.openoffice.setup.Util.InfoDir;

import java.io.File;
import java.lang.String;
import org.openoffice.setup.ResourceManager;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.Util.LogManager;
import org.openoffice.setup.Util.SystemManager;
import org.openoffice.setup.FX.controller.JavaFXTaskBase;

import javafx.scene.control.ProgressIndicator;

import org.openoffice.setup.Installer.Installer;
import org.openoffice.setup.SetupData.SetupDataProvider;


/**
 * @author is93718
 */

public class UninstallationOngoingDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des UninstallationOngoing-Dialoges";
    override public var titleText = "Deinstallation läuft";

    var helpFileName: String = "String_Helpfile_UninstallationOngoing";
    var mainLabel1Text: String = "Deinstallation läuft";
    var mainLabel2Text: String = "Deinstallation läuft";
    var mainLabel3Text: String = "Deinstallation läuft";
    var mainLabel1TextEnd: String = "Finishing Uninstallation";
    // var titleText: String = ResourceManager.getString("String_Prologue1");
    var dialogName: String = "UninstallationOngoing";  // only for Info

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.YELLOW;
                                     }

    var mainContent: MainContentInstallationOngoing = MainContentInstallationOngoing {
                                                          label1Text: mainLabel1Text;
                                                          label2Text: mainLabel2Text;
                                                          label3Text: mainLabel3Text;
                                                      }

    var buttonContent: ButtonContent = ButtonContent {
                                           visibleButtonBack: false
                                           visibleButtonNext: false
                                       };

    public override function getNext(): String {
        return "UninstallationDone";
    }

    public override function getPrevious(): String {
        return "";
    }

    public override function beforeShow(): Void {
        // creating list of packages to uninstall
        var uninstallPackages: Vector = new Vector();
        var packageData: PackageDescription = SetupDataProvider.getPackageDescription();
        PackageCollector.collectUninstallPackages(packageData, uninstallPackages);

        var sortedPackages: Vector = new Vector();
        PackageCollector.sortPackages(uninstallPackages, sortedPackages, "uninstall");
        Main.installData.setInstallPackages(sortedPackages);

        // collectPackages(packageData);

        var installer: Installer;
        // JavaFX Todo
        if ( not Main.installData.getOSType().startsWith("Windows")) {
            installer = InstallerFactory.getInstance();
            installer.preInstallationOngoing();
        }
    }

    function doUninstall(): Void {
        LogManager.setCommandsHeaderLine("Uninstallation");
        var uninstallPackages: Vector = Main.installData.getInstallPackages();
        var installer: Installer;

        // JavaFX Todo
        if ( not Main.installData.getOSType().startsWith("Windows")) {
            installer = InstallerFactory.getInstance();
        }

        for ( i in [0..uninstallPackages.size()-1] ) {

            if ( uninstallPackages.get(i) instanceof PackageDescription) {
                var packageData: PackageDescription = uninstallPackages.get(i) as PackageDescription;
                var localcount = i + 1;
                var progress = ProgressIndicator.computeProgress(uninstallPackages.size(), localcount);
                var prog: Integer = (progress * 100) as Integer;
                // var progress = ProgressIndicator.computeProgress(4, localcount);

                mainContent.publicLabel2Text = packageData.getPackageName();
                mainContent.publicLabel3Text = "{prog} %";
                mainContent.publicProgressCount = progress;
                FX.deferAction(function() { mainContent.updateContent() }); // update dialog

                if ( not Main.installData.getOSType().startsWith("Windows")) {
                    installer.uninstallPackage(packageData);
                }

                if ( Main.installData.isAbortedInstallation() ) {
                    break;
                }
            }
        }

        if ( Main.installData.isAbortedInstallation() ) {
            LogManager.setCommandsHeaderLine("Uninstallation aborted!");
            // undoing the uninstallation is not possible
        }

        if ( not Main.installData.getOSType().startsWith("Windows")) {
            installer.postUninstallationOngoing();
        }
    }

    public override function duringShow(): Void {

        JavaFXTaskBase {
            action: function() {
                // mainContent.showLabel2 = true;
                // localLabelText = "Loading components ...";

                doUninstall();
                afterShow(true);

                // finally setting the following dialog automatically
                FX.deferAction(function() {
                        Main.dialogController.setCurrentDialog(getNext())
                    }
                );  // attention: ui change from thread -> deferAction
            }
        }.start();
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;

        mainContent.publicLabel1Text = mainLabel1TextEnd;
        mainContent.publicLabel2Text = "";

        FX.deferAction(function() { mainContent.updateContent() }); // update dialog

        if ( not Main.installData.isAbortedInstallation() ) {
            if (( Main.installData.isTypicalInstallation() ) or ( Main.installData.isMaskedCompleteUninstallation() )) {
                InfoDir.removeUninstallationFiles();
            }
        }

        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

}
