/*
 * WelcomeDialog.fx
 *
 * Created on 23.07.2009, 14:30:27
 */

package org.openoffice.setup.FX.dialog;

import org.openoffice.setup.FX.Main;
import org.openoffice.setup.FX.dialog.Dialog;
import org.openoffice.setup.FX.controller.JavaFXTaskBase;
import org.openoffice.setup.FX.frame.*;
import javafx.scene.paint.Color;

import org.openoffice.setup.ResourceManager;

import java.lang.String;

import java.lang.System;
import org.openoffice.setup.Installer.Installer;
import org.openoffice.setup.Installer.InstallerFactory;
import org.openoffice.setup.SetupData.PackageDescription;
import org.openoffice.setup.SetupData.SetupDataProvider;
import org.openoffice.setup.Util.Controller;
import org.openoffice.setup.Util.Dumper;
import org.openoffice.setup.Util.ModuleCtrl;
import org.openoffice.setup.Util.SystemManager;

/**
 * @author is93718
 */

public class WelcomeDialog extends Dialog {

    override public var helpFileText = "Dieses ist die Hilfe des Welcome-Dialoges";
    override public var titleText = ResourceManager.getString("String_Prologue1");

    var helpFileName: String = "String_Helpfile_Prologue";
    var mainLabelText: String = "Herzlich Willkommen zur Installation dieses Produktes";
    var mainLabel2Text: String = "";
    var mainLabel3Text: String = "Version 12.10.09 A2";
    // var titleText: String = "Willkommen";
    var dialogName: String = "Welcome";  // only for Info

    var titleContent: TitleContent = TitleContent {
                                         titleTextString: titleText;
                                         titleLogoColor: Color.YELLOW;
                                     }

    var mainContent: MainContentWelcome = MainContentWelcome {
                                       labelText: mainLabelText;
                                       label2Text: mainLabel2Text;
                                       label3Text: mainLabel3Text;
                                   }

    var buttonContent: ButtonContent = ButtonContent {
                                           visibleButtonBack: false
                                       };

    public override function getNext(): String {
        if ( Main.installData.hideEula() ) {
            if ( Main.installData.isRootInstallation() ) {
                if ( Main.installData.olderVersionExists() ) {
                    return "ReadyToInstall";
                } else if ( Main.installData.sameVersionExists() ) {
                    return "ModuleSelection";
                } else {
                    return "SelectInstallationType";
                }
            } else {
                return "PathSelection";
            }
        } else {
            return "License";
        }
    }

    public override function getPrevious(): String {
        return "";
    }

    public override function beforeShow(): Void {
    }

    public override function duringShow(): Void {
        buttonContent.visibleButtonNext = false;
        buttonContent.visibleButtonCancel = false;

        JavaFXTaskBase {
            action: function() {
                // mainContent.showLabel2 = true;

                preInstall();

                mainContent.publicLabelText = "Herzlich Willkommen";
                mainContent.publicLabel2Text = "Alle Komponentent geladen";

                buttonContent.publicVisibleButtonNext = true;
                buttonContent.publicVisibleButtonCancel = true;

                FX.deferAction(function() { mainContent.updateContent() }); // update dialog
                FX.deferAction(function() { buttonContent.updateContent() }); // update dialog
            }
        }.start();
    }

    public override function afterShow(isNext: Boolean): Boolean {
        var repeatDialog: Boolean = false;
        return repeatDialog
    }

    public override function getName(): String {
        return dialogName;
    }

    public override function getHelpFileName(): String {
        return helpFileName;
    }

    public override function getHelpText(): String {
        return helpFileText;
    }

    public override function getTitleContent(): TitleContent {
        return titleContent
    }

    public override function getMainContent(): MainContent {
        return mainContent
    }

    public override function getButtonContent(): ButtonContent {
        return buttonContent
    }

    function preInstall(): Void {

        if ( not Main.installData.preInstallDone() ) {

            // Reading all xml files
            Controller.checkPackagePathExistence(Main.installData);
            Controller.checkPackageFormat(Main.installData);

            if (( Main.installData.getOSType().equalsIgnoreCase("SunOS") ) and ( Main.installData.isMultiLingual() )) {
                Controller.collectSystemLanguages(Main.installData);
            }

            var packageData: PackageDescription = SetupDataProvider.getPackageDescription();

            // JavaFX Todo
            if ( not Main.installData.getOSType().startsWith("Windows")) {
                var installer: Installer = InstallerFactory.getInstance();
                installer.preInstall(packageData);
            }

            Main.installData.setPreInstallDone(true);

            if ( SystemManager.logModuleStates() ) {
                Main.installData.setLogModuleStates(true);
            }

            if ( Main.installData.logModuleStates() ) {
                Dumper.logModuleStates(packageData, "Prologue Dialog");
            }

            if (( Main.installData.getOSType().equalsIgnoreCase("SunOS") ) and ( Main.installData.isMultiLingual() )) {
                ModuleCtrl.checkLanguagesPackages(packageData, Main.installData);

                // int count = installData.getPreselectedLanguages();
                // print("Number of preselected language packages: " + count);

                if ( Main.installData.getPreselectedLanguages() == 0 ) {
                    // Something misterious happened. Setting all languages again.
                    ModuleCtrl.setLanguagesPackages(packageData);
                }

                if ( Main.installData.logModuleStates() ) {
                    Dumper.logModuleStates(packageData, "Prologue Dialog Language Selection");
                }
            }

            if ( not Main.installData.isMultiLingual() ) {
                ModuleCtrl.setHiddenLanguageModuleDefaultSettings(packageData);

                if ( Main.installData.logModuleStates() ) {
                    Dumper.logModuleStates(packageData, "after setHiddenLanguageModuleDefaultSettings");
                }
            }

            if (( Main.installData.isRootInstallation() ) and ( Main.installData.getOSType().equalsIgnoreCase("SunOS") )) {
                // Check, if root has write access in /usr and /etc .
                // In sparse zones with imported directories this is not always the case.
                if ( Controller.reducedRootWritePrivileges() ) {
                    ModuleCtrl.setIgnoreNonRelocatablePackages(packageData);
                }

                if ( Main.installData.logModuleStates() ) {
                    Dumper.logModuleStates(packageData, "after setIgnoreNonRelocatablePackages");
                }
            }

            if ( Main.installData.isRootInstallation() ) {

                // Setting installation directory!
                var dir: String = "/";
                Main.installData.setInstallDir(dir);
                Main.installData.setInstallDefaultDir(Main.installData.getDefaultDir());

                Controller.checkForNewerVersion(Main.installData);

                // Check Write privileges in installation directory (installData.getInstallDefaultDir())
                // If the directory exists, is has to be tested, whether the user has write access
                dir = Main.installData.getInstallDefaultDir();

                if ( SystemManager.exists_directory(dir) ) {
                    if ( not Controller.createdSubDirectory(dir) ) {
                        System.err.println("ERROR: No write privileges inside directory: {dir}");
                        System.exit(1);
                    }
                }

                // If the directory does not exist, is has to be tested, whether the user can create it
                if ( not SystemManager.exists_directory(dir)) {
                    if ( not Controller.createdDirectory(dir) ) {
                        System.err.println("ERROR: No privileges to create directory: {dir}");
                        System.exit(1);
                    }
                }

                // Setting macro
                SetupDataProvider.setNewMacro("DIR", dir); // important for string replacement

                // Calculate available disc space
                var discSpace: Integer = SystemManager.calculateDiscSpace(dir);
                Main.installData.setAvailableDiscSpace(discSpace);

                if ( not Main.installData.databaseAnalyzed()) {
                    ModuleCtrl.defaultDatabaseAnalysis(Main.installData);
                    Main.installData.setDatabaseAnalyzed(true);
                }
            }
        }

    }

}
