/*
 * HelpDialogSkin.fx
 *
 * Created on 06.08.2009, 12:56:13
 */

package org.openoffice.setup.FX.helpdialog;

import javafx.scene.Group;
import javafx.scene.control.Skin;
import javafx.scene.effect.DropShadow;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.paint.Paint;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.scene.text.TextOrigin;

import javafx.ext.swing.SwingComponent;

import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import java.lang.String;
import java.awt.ComponentOrientation;
import java.awt.Dimension;

import org.openoffice.setup.FX.Main;

/**
 * @author is93718
 */

public class HelpDialogSkin extends Skin {
    def helpDialogControl = bind control as HelpDialog;

    public var borderFill: Paint = Color.LIGHTGRAY;
    public var headingFill: Paint = Color.WHITESMOKE;
    public var displayFill: Paint = Color.WHITE;
    public var footerFill: Paint = Color.WHITESMOKE;
    public var headingTextColor: Paint = Color.BLACK;
    public var displayTextColor: Paint = Color.BLACK;

    var textPane: JEditorPane;
    var scrollPane: JScrollPane;
    var displayTextComponent: SwingComponent;

    public function makeVisible() {
        (buttonConfirm.skin as TextButtonSkin).node.opacity = 1;
    }

    public var updateMainBox: String = "" on replace {
        textPane = helpDialogControl.createEditorPane(helpDialogControl.displayFile);
        textPane.setBackground(java.awt.Color.LIGHT_GRAY);

        scrollPane = new JScrollPane(textPane,
                                      ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                                      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED
                                  );

        scrollPane.setPreferredSize(new Dimension(330, 152));
        if ( Main.installData.useRtl() ) { scrollPane.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT); }

        displayTextComponent = SwingComponent.wrap(scrollPane);
        displayTextComponent.opacity = 0.9;
    }

    public var mouseEnteredColor: Paint on replace {
        (buttonConfirm.skin as TextButtonSkin).mouseEnteredColor = mouseEnteredColor;
    }

    public var mouseExitedColor: Paint on replace {
        (buttonConfirm.skin as TextButtonSkin).mouseExitedColor = mouseExitedColor;
    }

    public var mousePressedColor: Paint on replace {
        (buttonConfirm.skin as TextButtonSkin).mousePressedColor = mousePressedColor;
    }

    public override function contains(localX: Number, localY: Number) : Boolean {
        node.contains(localX, localY);
    }

    public override function intersects(localX: Number, localY: Number, localWidth: Number, localHeight: Number) : Boolean {
        node.intersects(localX, localY, localWidth, localHeight);
    }

    function fade(o1:Number): Number {
        if (o1 == 1) {
            helpDialogControl.visible = true;
            return 1;
        }
        if (o1 == 0) {
            helpDialogControl.visible = false;
            return 0;
        }
        return o1
    }

    // Objects that go into the component scene graph:
    def background: Rectangle = Rectangle {
        fill: bind borderFill
        height: bind headingBackground.height
            + displayBackground.height + footerBackground.height + 20
        width: bind headingBackground.width + 20
        arcHeight: 15
        arcWidth: 15
        opacity: 0.7
    }

    def headingBackground: Rectangle = Rectangle {
        x: 10
        y: 10
        width: bind if (helpDialogControl.windowWidth > headingText.layoutBounds.width + 20)
        helpDialogControl.windowWidth else headingText.layoutBounds.width + 20
        height: bind headingText.layoutBounds.height + 10
        fill: bind headingFill
        opacity: 0.9
    }

    def headingText: Text = Text{
        layoutY: bind headingBackground.layoutBounds.minY + 8
        // font: bind dialogFont
        textOrigin: TextOrigin.TOP
        fill: bind headingTextColor
        content: bind helpDialogControl.heading
        layoutX: bind headingBackground.layoutBounds.minX + 10
    }

    def displayBackground: Rectangle = Rectangle {
        x: 10
        layoutY: bind headingBackground.layoutBounds.maxY
        width: bind headingBackground.width
        height: bind displayTextComponent.layoutBounds.height
        fill: bind displayFill
        opacity: 0.5
    }

    def mainBox: VBox = VBox {
                             layoutX: bind headingBackground.layoutBounds.minY
                             layoutY: bind headingBackground.height + 10
                             content: bind displayTextComponent
                         };

    def footerBackground: Rectangle = Rectangle {
        x: 10
        layoutY: bind headingBackground.layoutBounds.maxY + displayBackground.height
        width: bind background.layoutBounds.width - 20
        height: bind buttonConfirm.layoutBounds.height - 15
        fill: footerFill
        opacity: 0.9
    }

    def buttonConfirm: TextButton = TextButton {
        layoutX: bind background.layoutBounds.maxX - buttonConfirm.layoutBounds.width - 100;
        layoutY: bind headingBackground.layoutBounds.maxY + displayBackground.height + 6
        displayText: bind helpDialogControl.buttonConfirm
        effect: DropShadow { }
        action: bind helpDialogControl.onConfirm
    }

    init {
        node = Group {
            content: [ background, headingBackground, headingText, displayBackground,
                       mainBox, footerBackground, buttonConfirm ]
            opacity: bind fade((buttonConfirm.skin as TextButtonSkin).node.opacity);
        }
    }
}

