/*
 * TextButtonSkin.fx
 *
 * Created on Apr 12, 2009, 2:58:20 PM
 */

package org.openoffice.setup.FX.helpdialog;

import javafx.animation.transition.FadeTransition;
import javafx.scene.control.Skin;
import javafx.scene.Cursor;
import javafx.scene.Group;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.paint.Paint;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.scene.text.TextOrigin;

import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;

/**
 * @author Gail
 */

public class TextButtonSkin extends Skin {

    def textButtonControl = bind control as TextButton;

    // Skinnable properties with their default values
    public var fillColor: Paint = Color.LIGHTBLUE;
    public var outlineColor: Paint = Color.BLACK;
    public var mouseEnteredColor: Paint = Color.LIGHTSLATEGRAY;
    public var mouseExitedColor: Paint = Color.LIGHTBLUE;
    public var mousePressedColor: Paint = Color.GRAY;
    public var mouseReleasedColor: Paint = Color.GRAY;
    public var textColor: Paint = Color.BLACK;
//    public var buttonFont: Font = Font {
//        size: 14
//        name: "Arial Bold"
//    }

    def buttonGradient: LinearGradient = LinearGradient {
        startX: 1
        startY: 0
        endX: 1
        endY: 1
        stops: [
            Stop {
                offset: 0
                color: Color.LIGHTGREY
            },
            Stop {
                offset: 1
                color: Color.LIGHTSLATEGRAY
            }
        ]
    }

    def button: Rectangle = Rectangle {
        fill: buttonGradient
        // fill: bind fillColor
        stroke: bind outlineColor
        height: bind text.layoutBounds.height + 10;
        width: bind text.layoutBounds.width + 15;
        arcWidth: 10
        arcHeight: 10
    }

    def text = Text {
        x: 10
        y: 8
//        font: bind buttonFont
        textOrigin: TextOrigin.TOP
        fill: bind textColor
        content: bind textButtonControl.displayText
    }

    public override function contains(localX: Number, localY: Number) : Boolean {
        node.contains(localX, localY);
    }
    public override function intersects(localX: Number, localY: Number, localWidth: Number, localHeight: Number) : Boolean {
        node.intersects(localX, localY, localWidth, localHeight);
    }


    init {
        // behavior = TextButtonBehavior { };
        behavior = null;

        node = Group {
        //this Group represents a button that is pressed
            cursor: Cursor.HAND
            onMouseEntered: function (evt: MouseEvent): Void {
                fillColor = mouseEnteredColor;
            }
            onMouseExited: function (evt: MouseEvent): Void {
                fillColor = mouseExitedColor;
            }
            onMousePressed: function(evt: MouseEvent):Void {
                if(evt.button == MouseButton.PRIMARY){
                    fillColor = mousePressedColor;
                }
            }
            onMouseReleased: function(evt: MouseEvent):Void {
                if(evt.button == MouseButton.PRIMARY){
                    fillColor = mouseReleasedColor;
                    textButtonControl.action();
                    def fade = FadeTransition {
                        node: node
                        duration: 1.5s
                        fromValue: 1
                        toValue: .0
                    }
                    if (textButtonControl.disappearOnAction) fade.play();
                }
            }
            content: [ button, text ]
        }
    }
}
